
<!-- start comment -->
<div class="comment"><span class="commentmarker">{{COMMENT:</span> 
; &lt;script&gt;&quot;hello&quot;&lt;/script&gt;
<span class="commentmarker">END_COMMENT}}:</span>
</div>
<!-- stop comment -->

<!-- start comment -->
<div class="comment"><span class="commentmarker">{{COMMENT:</span> 

<p>
This is &lt;script&gt;alert&lt;/script&gt; a comment \emph
</p>

<span class="commentmarker">END_COMMENT}}:</span>
</div>
<!-- stop comment -->
